# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Definition of Invenio Fabric API. To make all the library tasks available
in a fabfile.py you just have to have the following import in the top
of your file::

  from inveniofab.api import *
"""

from inveniofab.apache import apache_start, apache_restart, apache_stop
from inveniofab.env import env_defaults, env_get, env_create, env_override, \
    env_make_name
from inveniofab.git import repo_update, repo_setup, repo_prepare, \
    repo_install, repo_make, repo_configure, repo_check
from inveniofab.invenio import invenio_conf, invenio_create_demosite, \
    invenio_createdb, invenio_upgrade
from inveniofab.mysql import mysql_dropdb, mysql_createdb, mysql_load, \
    mysql_dump, mysql_copy
from inveniofab.test import test_load, test_dump, test_clean, test_reset_admin
from inveniofab.venv import venv_create, venv_dump, venv_load, venv_drop, \
    venv_requirements
from inveniofab.compound import bootstrap, install, dump, load, drop
from inveniofab.devserver import devserver_conf, devserver_install_flask


# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Library tasks for configuring and running MySQL for Invenio.
"""

from __future__ import with_statement
from fabric.api import puts, task, env, local, abort, settings, hide
from fabric.colors import red, cyan
from fabric.contrib.console import confirm
from inveniofab.env import env_get
from inveniofab.utils import prompt_and_check
import os

@task
def mysql_dropdb(stored_answers=None):
    """
    Drop database and user
    """
    with hide('commands'):
        puts(cyan(">>> Dropping database and user ..." % env))

        answers = prompt_and_check([
            ("MySQL admin user:", "user"),
            ("MySQL admin password:", "password")
        ], mysql_admin_check(env.CFG_DATABASE_HOST, env.CFG_DATABASE_PORT),
        cache_key="mysql://%s:%s" % (env.CFG_DATABASE_HOST, env.CFG_DATABASE_PORT),
        stored_answers=stored_answers)

        # Escape quote characters
        answers['password'] = answers['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')
        user_pw = '-u %(user)s --password=%(password)s' % answers if answers['password'] else '-u %(user)s' % answers

        ctx = {
            'user_pw': user_pw,
            'host': env.CFG_DATABASE_HOST,
            'name' :  env.CFG_DATABASE_NAME,
            'user' :  env.CFG_DATABASE_USER,
            'password' : env.CFG_DATABASE_PASS,
            'port': env.CFG_DATABASE_PORT,
        }

        ctx['password'] = ctx['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')

        local('mysql %(user_pw)s -h %(host)s -P %(port)s -e "DROP DATABASE IF EXISTS %(name)s"' % ctx)
        with settings(warn_only=True):
            local('mysql %(user_pw)s -h %(host)s -P %(port)s -e "REVOKE ALL PRIVILEGES ON %(name)s.* FROM %(user)s@localhost"' % ctx)
        local('mysqladmin %(user_pw)s -h %(host)s -P %(port)s flush-privileges' % ctx)


@task
def mysql_createdb(stored_answers=None):
    """
    Create database and user
    """
    with hide('commands'):
        puts(cyan(">>> Creating database and user ..." % env))

        answers = prompt_and_check([
            ("MySQL admin user:", "user"),
            ("MySQL admin password:", "password")
        ], mysql_admin_check(env.CFG_DATABASE_HOST, env.CFG_DATABASE_PORT),
        cache_key="mysql://%s:%s" % (env.CFG_DATABASE_HOST, env.CFG_DATABASE_PORT),
        stored_answers=stored_answers)

        # Escape quote characters
        answers['password'] = answers['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')
        user_pw = '-u %(user)s --password=%(password)s' % answers if answers['password'] else '-u %(user)s' % answers

        ctx = {
            'user_pw': user_pw,
            'host' :  env.CFG_DATABASE_HOST,
            'name' :  env.CFG_DATABASE_NAME,
            'user' :  env.CFG_DATABASE_USER,
            'password' : env.CFG_DATABASE_PASS,
            'port': env.CFG_DATABASE_PORT,
        }

        ctx['password'] = ctx['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')

        # Run commands
        local('mysql %(user_pw)s -h %(host)s -P %(port)s -e "CREATE DATABASE IF NOT EXISTS %(name)s DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci"' % ctx)
        local('mysql %(user_pw)s -h %(host)s -P %(port)s -e "GRANT ALL PRIVILEGES ON %(name)s.* TO %(user)s@localhost IDENTIFIED BY \'%(password)s\';"' % ctx)
        local('mysqladmin %(user_pw)s -h %(host)s -P %(port)s flush-privileges' % ctx)


@task
def mysql_dump(outputdir=None):
    """
    Dump database to file
    """
    with hide('commands'):
        puts(cyan(">>> Dumping database ..." % env))

        answers = {
            'user' : env.CFG_DATABASE_USER,
            'password' : env.CFG_DATABASE_PASS,
        }

        # Escape quote characters
        answers['password'] = answers['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')

        if not outputdir:
            outputdir = env.CFG_DATABASE_DUMPDIR

        if not os.path.exists(outputdir):
            abort(red("Output directory %s does not exists" % outputdir))

        user_pw = '-u %(user)s --password=%(password)s' % answers if answers['password'] else '-u %(user)s' % answers
        outfile = os.path.join(outputdir, "%s.sql" % env.CFG_DATABASE_NAME)
        outfile_gz = "%s.gz" % outfile

        for f in [outfile, outfile_gz]:
            if os.path.exists(f):
                res = confirm("Remove existing DB dump in %s ?" % f)
                if not res:
                    abort(red("Cannot continue") % env)
                else:
                    local("rm -Rf %s" % f)

        ctx = {
            'user_pw': user_pw,
            'host' :  env.CFG_DATABASE_HOST,
            'name' :  env.CFG_DATABASE_NAME,
            'port': env.CFG_DATABASE_PORT,
            'outfile_gz': outfile_gz,
            'outfile': outfile,
        }

        # Run commands
        local('mysqldump %(user_pw)s -h %(host)s -P %(port)s --skip-opt '
              '--add-drop-table --add-locks --create-options --quick '
              '--extended-insert --set-charset --disable-keys %(name)s '
              '| gzip -c > %(outfile_gz)s' % ctx)


@task
def mysql_load(dumpfile=None, stored_answers=None):
    """
    Load MySQL dump file
    """
    with hide('commands'):
        puts(cyan(">>> Loading database dump..."))

        if not dumpfile:
            dumpfile = os.path.join(env.CFG_DATABASE_DUMPDIR, "%s.sql.gz" % env.CFG_DATABASE_NAME)

        if not os.path.exists(dumpfile):
            abort("File %s does not exists." % dumpfile)

        answers = prompt_and_check([
            ("MySQL admin user:", "user"),
            ("MySQL admin password:", "password")
        ], mysql_admin_check(env.CFG_DATABASE_HOST, env.CFG_DATABASE_PORT),
        cache_key="mysql://%s:%s" % (env.CFG_DATABASE_HOST, env.CFG_DATABASE_PORT),
        stored_answers=stored_answers)

        # Escape quote characters
        answers['password'] = answers['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')
        user_pw = '-u %(user)s --password=%(password)s' % answers if answers['password'] else '-u %(user)s' % answers

        if dumpfile.endswith(".gz"):
            dumpfile_stream = "gunzip -c %s" % dumpfile
        else:
            dumpfile_stream = "cat %s" % dumpfile

        ctx = {
            'user_pw': user_pw,
            'host' :  env.CFG_DATABASE_HOST,
            'name' :  env.CFG_DATABASE_NAME,
            'user' :  env.CFG_DATABASE_USER,
            'password' : env.CFG_DATABASE_PASS,
            'port': env.CFG_DATABASE_PORT,
            'dumpfile': dumpfile,
            'dumpfile_stream': dumpfile_stream,
        }

        # Escape quote characters
        ctx['password'] = ctx['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')

        if confirm("This will erease all data in the existing database. Are you sure you want to load %(dumpfile)s?" % ctx):
            mysql_dropdb(stored_answers=answers)
            mysql_createdb(stored_answers=answers)

            local('%(dumpfile_stream)s | mysql %(user_pw)s -h %(host)s -P %(port)s -f %(name)s' % ctx)

        return dumpfile


@task
def mysql_copy(from_env):
    """
    Copy database from latest available dump.

    Currently it is assumed that the dump file is accessible on the
    same path on both environment host systems. This usually means that
    the dumps are stored on a shared network storage.
    """
    with hide('commands'):
        to_env = env
        from_env = env_get(from_env)

        answers_from = prompt_and_check([
            ("FROM MySQL admin user:", "user"),
            ("FROM MySQL admin password:", "password")
        ], mysql_admin_check(from_env.CFG_DATABASE_HOST, from_env.CFG_DATABASE_PORT))

        answers_to = prompt_and_check([
            ("TO MySQL admin user:", "user"),
            ("TO MySQL admin password:", "password")
        ], mysql_admin_check(to_env.CFG_DATABASE_HOST, to_env.CFG_DATABASE_PORT))

        # Escape quote characters
        answers_from['password'] = answers_from['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')
        answers_to['password'] = answers_to['password'].replace("'", "\'").replace('"', '\"').replace('$', '\\$')

        user_pw_from = '-u %(user)s --password=%(password)s' % answers_from if answers_from['password'] else '-u %(user)s' % answers_from
        user_pw_to = '-u %(user)s --password=%(password)s' % answers_to if answers_to['password'] else '-u %(user)s' % answers_to

        ctx = {
            'user_pw_from': user_pw_from,
            'user_pw_to': user_pw_to,
            'from_host' :  from_env.CFG_DATABASE_HOST,
            'from_name' :  from_env.CFG_DATABASE_NAME,
            'from_user' :  from_env.CFG_DATABASE_USER,
            'from_password' : from_env.CFG_DATABASE_PASS,
            'from_port': from_env.CFG_DATABASE_PORT,
            'to_host' :  to_env.CFG_DATABASE_HOST,
            'to_name' :  to_env.CFG_DATABASE_NAME,
            'to_user' :  to_env.CFG_DATABASE_USER,
            'to_password' : to_env.CFG_DATABASE_PASS,
            'to_port': to_env.CFG_DATABASE_PORT,
        }


        puts(">>> You are about to copy:")
        puts(">>>   %(from_user)s@%(from_host)s:%(from_port)s/%(from_name)s" % ctx)
        puts(">>> to:")
        puts(">>>   %(to_user)s@%(to_host)s:%(to_port)s/%(to_name)s" % ctx)

        if confirm("This will erease all data in the latter database and may impact performance on system being copied from. Are you sure you want to proceed?"):
            mysql_dropdb(stored_answers=answers_to)
            mysql_createdb(stored_answers=answers_to)

        local('mysqldump %(user_pw_from)s -h %(from_host)s -P %(from_port)s -f %(from_name)s | mysql %(user_pw_to)s -h %(to_host)s -P %(to_port)s -f %(to_name)s' % ctx)


#
# Helpers
#

def mysql_admin_check(host, port):
    def _mysql_admin_check(answers):
        ctx = {'host': host, 'port': port}
        ctx.update(answers)
        try:
            if answers['password']:
                res = local("""mysql -u %(user)s --password=%(password)s -h %(host)s -P %(port)s -e 'SELECT 1'""" % ctx)
            else:
                res = local("""mysql -u %(user)s -h %(host)s -P %(port)s -e 'SELECT 1'""" % ctx)

            return True
        except SystemExit:
            puts(red("MySQL admin user/password is not valid. Please try again."))
            return False

    return _mysql_admin_check



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Task for checking out source code from a repository, and running configure,
make on it.
"""

from fabric.api import task, puts, env, settings, local, abort
from fabric.contrib.console import confirm
from fabric.colors import cyan, red
import os

@task
def repo_configure(repo):
    """ Configure repository """
    topsrcdir = repo_check(repo, workdir=True)

    puts(cyan(">>> Configuring repository: %s ..." % repo))

    ctx = {
        'topsrcdir': topsrcdir,
    }
    ctx.update(env)

    if os.path.exists(os.path.join(topsrcdir, "configure")):
        with settings(warn_only=True):
            local("cd %(topsrcdir)s && make -s clean" % ctx)
    local("cd %(topsrcdir)s && ./configure --prefix=%(CFG_INVENIO_PREFIX)s "
          "--with-python=%(CFG_INVENIO_PREFIX)s/bin/python" % ctx)
    local("cd %(topsrcdir)s && make -s clean" % ctx)


@task
def repo_make(repo, *targets):
    """ Run make in repository """
    topsrcdir = repo_check(repo, workdir=True)

    if not targets:
        try:
            targets = dict(env.CFG_INVENIO_REPOS)[repo]['targets']
        except KeyError:
            abort(red("No default targets found for repository %s" % repo))

    puts(cyan(">>> Running make for %s targets: %s ..." % (repo, " ".join(targets))))

    ctx = {
        'topsrcdir': topsrcdir,
        'targets' : " ".join(targets),
    }
    ctx.update(env)

    local("cd %(topsrcdir)s && make %(targets)s" % ctx)


@task
def repo_install(repo=None, targets_key='install_targets'):
    """ Run configure and make """
    repo_all_configure_make(repo, targets_key)


@task
def repo_prepare(repo=None):
    """ Prepare source code after fresh checkout """
    topsrcdir = repo_check(repo, workdir=True)
    local("cd %s; aclocal && automake -a && autoconf -f" % topsrcdir)


@task
def repo_setup(repo, ref):
    """ Clone repository """
    puts(cyan(">>> Setting up repository %s with ref %s..." % (repo, ref)))

    topsrcdir = repo_check(repo, check_path=False, workdir=False)
    workdir = repo_check(repo, check_path=False, workdir=True)
    gitdir = os.path.join(topsrcdir, '.git')

    if not os.path.exists(env.CFG_SRCDIR):
        res = confirm("Create repository root %s?" % env.CFG_SRCDIR)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("mkdir -p %s" % env.CFG_SRCDIR)

    if not os.path.exists(gitdir) and os.path.exists(topsrcdir):
        res = confirm("Remove %s (it does not seem to be a git repository)?" % topsrcdir)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("rm -Rf %s" % topsrcdir)

    if not os.path.exists(gitdir):
        git_clone(repo)
    if not os.path.exists(workdir):
        git_newworkdir(repo)
    git_checkout(repo, ref)
    repo_prepare(repo)


@task
def repo_update(**kwargs):
    """ Pull repository updates """
    # Check repositories to update/setup
    repo_refs = []
    for repo, info in env.CFG_INVENIO_REPOS:
        if repo in kwargs:
            ref = kwargs[repo]
            if not ref:
                ref = None
            repo_refs.append(repo, ref)
            del kwargs[repo]
        else:
            try:
                ref = info['ref']
            except KeyError:
                ref = None
            repo_refs.append((repo, ref))

    if kwargs:
        for repo, ref in kwargs.items():
            abort(red("Invalid repository %s" % repo))

    # Run update
    for repo, ref in repo_refs:
        topsrcdir = repo_check(repo, check_path=False, workdir=True)
        gitdir = os.path.join(topsrcdir, '.git')

        if not os.path.exists(gitdir):
            repo_setup(repo, ref)
        else:
            if ref:
                puts(cyan(">>> Updating repository %s to ref %s..." % (repo, ref)))
                git_fetch(repo)
                git_checkout(repo, ref)
            else:
                commit_id = git_describe(repo)
                puts(cyan(">>> No ref specified for repository %s (currently at HEAD, commit %s) ..." % (repo, commit_id)))

#
# Helpers
#

def repo_check(repo, check_path=True, workdir=False):
    all_repos = [x[0] for x in env.CFG_INVENIO_REPOS]

    if repo not in all_repos:
        abort(red("%s is not a valid repository." % repo))

    repo_path = os.path.join(env.CFG_SRCWORKDIR if workdir and env.WITH_WORKDIR
                             else env.CFG_SRCDIR, repo)

    if check_path and not os.path.exists(repo_path):
        abort(red("Repository does not exists %s" % repo_path))

    return repo_path


def repo_all_configure_make(repo, target_key):
    if repo:
        repo_check(repo)
        repos = [(repo, dict(env.CFG_INVENIO_REPOS)[repo])]
    else:
        repos = env.CFG_INVENIO_REPOS

    for r, info in repos:
        repo_configure(r)

    for r, info in repos:
        try:
            repo_make(r, *info[target_key])
        except KeyError:
            repo_make(r, *['all', 'install'])

#
# Git helpers
#

def git_describe(repo):
    topsrcdir = repo_check(repo, workdir=True)
    version_gen = os.path.join(topsrcdir, 'git-version-gen')

    if os.path.exists(version_gen):
        output = local("cd %s; %s" % (topsrcdir, version_gen), capture=True)
    else:
        output = local("cd %s; git describe --always --abbrev=4 HEAD" % topsrcdir, capture=True)
    return output


def git_show_ref(repo):
    topsrcdir = repo_check(repo, workdir=True)
    local("cd %s; git show-ref" % topsrcdir)


def git_reset(repo, ref):
    topsrcdir = repo_check(repo, workdir=True)
    ctx = {
        'topsrcdir': topsrcdir,
        'ref': ref,
    }
    ctx.update(env)
    local("cd %(topsrcdir)s; git reset --hard %(ref)s " % ctx)


def git_newworkdir(repo):
    if env.WITH_WORKDIR and env.CFG_SRCDIR != env.CFG_SRCWORKDIR:
        srcdir = repo_check(repo, workdir=False, check_path=False)
        srcworkdir = repo_check(repo, workdir=True, check_path=False)

        ctx = {
            'srcdir': srcdir,
            'srcworkdir': srcworkdir,
        }
        ctx.update(env)

        local("%(CFG_INVENIO_PREFIX)s/bin/git-new-workdir %(srcdir)s %(srcworkdir)s" % ctx)


def git_checkout(repo, ref):
    topsrcdir = repo_check(repo, workdir=True)

    ctx = {
        'topsrcdir': topsrcdir,
        'ref': ref,
    }
    ctx.update(env)

    # Stash away any uncomitted changes
    local("cd %(topsrcdir)s; if ! git diff-index --quiet HEAD --; then git stash; fi; git checkout -f %(ref)s" % ctx)


def git_clone(repo):
    topsrcdir = repo_check(repo, check_path=False, workdir=False)

    try:
        repo_url = dict(env.CFG_INVENIO_REPOS)[repo]['repository']
    except KeyError:
        abort(red("Repository URL for %s not defined" % repo))

    basename = os.path.basename(topsrcdir)
    parent = os.path.dirname(topsrcdir)

    if os.path.exists(topsrcdir):
        res = confirm("Remove existing source code in %s ?" % topsrcdir)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("rm -Rf %s" % topsrcdir)
    else:
        if not os.path.exists(parent):
            local("mkdir -p %s" % parent)

    ctx = {
        'basename': basename,
        'parent': parent,
        'topsrcdir': topsrcdir,
        'url': repo_url,
    }

    local("cd %(parent)s; git clone %(url)s %(basename)s " % ctx)


def git_fetch(repo):
    topsrcdir = repo_check(repo)
    local("cd %s; git fetch origin" % topsrcdir)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Compound tasks to perform bigger operations like bootstrapping Invenio
"""

from fabric.api import task, env
from fabric.colors import cyan
from fabric.contrib.console import confirm
from inveniofab.devserver import devserver_conf
from inveniofab.git import repo_update, repo_install
from inveniofab.invenio import invenio_conf, invenio_createdb, \
    invenio_upgrade
from inveniofab.mysql import mysql_createdb, mysql_dump, mysql_load, \
    mysql_dropdb
from inveniofab.venv import venv_create, venv_requirements, venv_dump, \
    venv_load, venv_drop


@task
def bootstrap(with_db=True, quite=False, **kwargs):
    """ Bootstrap Invenio installation """

    def _confirm_step(func, *args, **kwargs):
        if quite or confirm(cyan("Run step %s?" % func.__name__)):
            func(*args, **kwargs)

    if with_db:
        _confirm_step(mysql_dropdb)
        _confirm_step(mysql_createdb)
    _confirm_step(venv_create)
    _confirm_step(repo_update, **kwargs)
    _confirm_step(venv_requirements)
    _confirm_step(repo_install, targets_key='bootstrap_targets')
    _confirm_step(invenio_conf)
    if env.WITH_DEVSERVER:
        _confirm_step(devserver_conf)
    if with_db:
        _confirm_step(invenio_createdb)


@task
def install(quite=False, **kwargs):
    """ Install changes """

    def _confirm_step(func, *args, **kwargs):
        if quite or confirm("Run step %s?" % func.__name__):
            func(*args, **kwargs)

    _confirm_step(repo_update, **kwargs)
    _confirm_step(repo_install, targets_key='deploy_targets')
    _confirm_step(invenio_conf)
    _confirm_step(invenio_upgrade)


@task
def dump():
    """ Archive installation """
    mysql_dump()
    venv_dump()


@task
def load():
    """ Load archived installation """
    venv_load()
    mysql_load()


@task
def drop():
    """ Remove installation """
    venv_drop()
    mysql_dropdb()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Fabric library tasks for bootstrapping, installing, running and copying Invenio

Usage
=====
Invenio Fabric is only a library of Fabric tasks, so you need to create your
own ``fabfile.py``. For a complete example for Atlantis Institute of Fictive 
Science please see ``examples/atlantis/``. A basic fabfile would allow your to 
run commands like: 

  * ``fab loc bootstrap``
  
Usually your fabfile for your Invenio installation would need only to define 
tasks for defining your environments (e.g. production, integration, testing).
"""


# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Task for creating and installing a Python virtual environment.
"""

from fabric.api import task, puts, env, local, abort
from fabric.contrib.console import confirm
from fabric.colors import red, cyan
from inveniofab.utils import write_template, python_version
import os


@task
def venv_requirements():
    """ Install Python requirements """
    requirements = []
    for repo, info in env.CFG_INVENIO_REPOS:
        requirements.extend([(repo, x) for x in info.get('requirements', [])])

    if not requirements:
        puts(cyan(">>> No requirements defined..."))
        return

    puts(cyan(">>> Installing requirements..." % env))

    pyver = python_version()

    for repo, reqfile in requirements:
        reqfile = reqfile % env
        reqpath = os.path.join(env.CFG_INVENIO_PREFIX, "%s_%s" % (repo, os.path.basename(reqfile)))

        ctx = {'reqpath': reqpath, 'reqfile': reqfile, 'pyver': pyver}
        ctx.update(env)

        puts(">>> Writing requirements to %(reqpath)s ..." % ctx)
        write_template(reqpath, ctx, tpl_file=reqfile)

        puts(">>> Installing requirements from %(reqpath)s ..." % ctx)
        cmds = [env.ACTIVATE] if env.WITH_VIRTUALENV else []
        cmds.append("pip install -r %(reqpath)s")
        local(" && ".join(cmds) % ctx)


@task
def venv_dump():
    """ Archive a virtualenv """
    puts(cyan(">>> Creating archive of virutalenv in %(CFG_INVENIO_PREFIX)s..." % env))

    ctx = {
        'dirname' : os.path.dirname(env.CFG_INVENIO_PREFIX),
        'basename' : os.path.basename(env.CFG_INVENIO_PREFIX),
    }

    archive_file = "%(dirname)s/%(basename)s.tar.gz" % ctx
    if os.path.exists(archive_file):
        res = confirm("Existing archive already exists - remove?")
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("rm -Rf %s" % archive_file)

    cmds = [
        "cd %(dirname)s",
        "tar -cvzf %(basename)s.tar.gz %(basename)s",
    ]
    local(" && ".join(cmds) % ctx)


@task
def venv_load():
    """ Load an archived virtualenv """
    puts(cyan(">>> Loading archived virutalenv..."))

    ctx = {
        'dirname' : os.path.dirname(env.CFG_INVENIO_PREFIX),
        'basename' : os.path.basename(env.CFG_INVENIO_PREFIX),
    }

    archive_file = "%(dirname)s/%(basename)s.tar.gz" % ctx
    if not os.path.exists(archive_file):
        abort(red("Archived virtualenv does not exists - cannot continue") % env)

    # Remove previous installation
    if os.path.exists(env.CFG_INVENIO_PREFIX):
        res = confirm("Remove installation in %(CFG_INVENIO_PREFIX)s ?" % env)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("sudo rm -Rf %(CFG_INVENIO_PREFIX)s" % env)

    cmds = [
        "cd %(dirname)s",
        "tar -xvzf %(basename)s.tar.gz",
    ]
    local(" && ".join(cmds) % ctx)


@task
def venv_create():
    """ Create virtualenv environment """
    # Checks
    if 'CFG_INVENIO_PREFIX' not in env:
        abort(red("CFG_INVENIO_PREFIX is not specified in env.") % env)

    puts(cyan(">>> Creating virutalenv in %(CFG_INVENIO_PREFIX)s..." % env))

    # Remove previous installation
    if os.path.exists(env.CFG_INVENIO_PREFIX):
        res = confirm("Remove installation in %(CFG_INVENIO_PREFIX)s ?" % env)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("sudo rm -Rf %(CFG_INVENIO_PREFIX)s" % env)

    # Create virtual environment
    dirname = os.path.dirname(env.CFG_INVENIO_PREFIX)
    basename = os.path.basename(env.CFG_INVENIO_PREFIX)

    local("mkdir -p %s" % dirname)
    local("cd %s && virtualenv -p %s %s" % (dirname, env.PYTHON, basename))

    # Create needed symboic links
    pyver = python_version()
    local("mkdir -p %(CFG_INVENIO_PREFIX)s/lib/python/invenio" % env)
    local(("mkdir -p %(CFG_INVENIO_PREFIX)s/lib/python" + pyver + "/site-packages") % env)
    local(("ln -s %(CFG_INVENIO_PREFIX)s/lib/python/invenio %(CFG_INVENIO_PREFIX)s/lib/python" + pyver + "/site-packages/invenio") % env)

    # Write extras into the activate script
    write_template(os.path.join(env.CFG_INVENIO_PREFIX, 'bin/activate'), env, tpl_file='activate-profile.tpl', append=True, mark="ACTIVATE_PROFILE")

    # Install devscripts
    if env.WITH_DEVSCRIPTS:
        puts(">>> Installing invenio-devscripts...")
        local("cd %(CFG_INVENIO_PREFIX)s && git clone https://github.com/tiborsimko/invenio-devscripts.git" % env)
        local("cd %(CFG_INVENIO_PREFIX)s && mv invenio-devscripts/* bin/" % env)

    if env.WITH_WORKDIR:
        puts(">>> Installing git-new-workdir...")
        local('wget -O %(CFG_INVENIO_PREFIX)s/bin/git-new-workdir "http://repo.or.cz/w/git.git/blob_plain/HEAD:/contrib/workdir/git-new-workdir"' % env)
        local("chmod +x %(CFG_INVENIO_PREFIX)s/bin/git-new-workdir" % env)

    # OpenOffice temporary directory
    local("mkdir -p %(CFG_INVENIO_PREFIX)s/var/tmp/ooffice-tmp-files" % env)
    local("sudo chown -R nobody %(CFG_INVENIO_PREFIX)s/var/tmp/ooffice-tmp-files" % env)
    local("sudo chmod -R 755 %(CFG_INVENIO_PREFIX)s/var/tmp/ooffice-tmp-files" % env)


@task
def venv_drop():
    """ Drop virtualenv environment """
    # Checks
    if 'CFG_INVENIO_PREFIX' not in env:
        abort(red("CFG_INVENIO_PREFIX is not specified in env.") % env)

    puts(cyan(">>> Dropping virutalenv in %(CFG_INVENIO_PREFIX)s..." % env))

    # Remove previous installation
    if os.path.exists(env.CFG_INVENIO_PREFIX):
        res = confirm("Remove installation in %(CFG_INVENIO_PREFIX)s ?" % env)
        if not res:
            abort(red("Cannot continue") % env)
        else:
            local("sudo rm -Rf %(CFG_INVENIO_PREFIX)s" % env)
    else:
        puts(">>> Nothing to remove - %(CFG_INVENIO_PREFIX)s does not exists..." % env)




# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Invenio Fabric tasks for Apache
"""

from fabric.api import roles, sudo, task, env

@roles('web')
@task
def apache_start():
    """ Restart Apache """
    sudo("%(CFG_INVENIO_APACHECTL)s start" % env)


@roles('web')
@task
def apache_restart():
    """ Restart Apache """
    sudo("%(CFG_INVENIO_APACHECTL)s configtest" % env)
    sudo("%(CFG_INVENIO_APACHECTL)s graceful" % env)


@roles('web')
@task
def apache_stop():
    """ Restart Apache """
    sudo("%(CFG_INVENIO_APACHECTL)s stop" % env)


# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Task to work with devserver
"""

from fabric.api import task, puts, env, local, warn
from fabric.colors import cyan, red
from inveniofab.utils import write_template, python_version
from jinja2.exceptions import TemplateNotFound
import os
import stat

@task
def devserver_conf():
    """ Upload and update Invenio configuration """
    puts(cyan(">>> Configuring invenio-devserver..." % env))

    pyver = python_version()

    local_file = 'config_local.py.tpl'
    local_remote = os.path.join(env.CFG_INVENIO_PREFIX, 'lib/python%s/site-packages/config_local.py' % pyver)

    puts(">>> Writing config_local.py to %s ..." % local_remote)
    try:
        write_template(local_remote, env, tpl_file=local_file)
    except TemplateNotFound:
        warn(red("Could not find template %s" % local_file))


@task
def devserver_install_flask():
    """ Install a Flask devserver """
    puts(cyan(">>> Configuring Flask devserver..." % env))
    
    local_file = 'rundevserver.py.tpl'
    local_remote = os.path.join(env.CFG_INVENIO_PREFIX, 'bin/rundevserver.py')

    puts(">>> Writing rundevserver.py to %s ..." % local_remote)
    try:
        write_template(local_remote, env, tpl_file=local_file)
        local("chmod a+x %s" % local_remote)
    except TemplateNotFound:
        warn(red("Could not find template %s" % local_file))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Tasks for creating and loading a test fixture package to easily run tests
against a known state. 
"""

from fabric.api import task, puts, local, env
from fabric.colors import cyan
from inveniofab.mysql import mysql_dump, mysql_load
import os


@task
def test_clean():
    """ Clean Invenio logs and temporary files """
    puts(cyan(">>> Cleaning Invenio installation..." % env))
    local("sudo rm -Rf %(CFG_INVENIO_PREFIX)s/var/tmp/ooffice-tmp-files/*" % env)
    local("rm -Rf %(CFG_INVENIO_PREFIX)s/var/tmp-shared/*" % env)
    local("rm -Rf %(CFG_INVENIO_PREFIX)s/var/tmp/*" % env)
    local("rm -Rf %(CFG_INVENIO_PREFIX)s/var/log/*" % env)
    local("rm -Rf %(CFG_INVENIO_PREFIX)s/var/cache/*" % env)
    local("""echo "TRUNCATE schTASK;" | %(CFG_INVENIO_PREFIX)s/bin/dbexec""" % env)
    local("""echo "TRUNCATE session;" | %(CFG_INVENIO_PREFIX)s/bin/dbexec""" % env)


@task
def test_dump():
    """ Dump a test environment """
    test_reset_admin()
    test_clean()

    puts(cyan(">>> Creating test package..." % env))

    local("rm -Rf %(CFG_INVENIO_PREFIX)s/tests/" % env)
    local("mkdir -p %(CFG_INVENIO_PREFIX)s/tests/var/" % env)
    local("mkdir -p %(CFG_INVENIO_PREFIX)s/tests/etc/" % env)
    local("sudo rsync --delete -rLptgoDv %(CFG_INVENIO_PREFIX)s/var/ %(CFG_INVENIO_PREFIX)s/tests/var/" % env)
    local("sudo rsync --delete -rLptgoDv %(CFG_INVENIO_PREFIX)s/etc/ %(CFG_INVENIO_PREFIX)s/tests/etc/" % env)

    mysql_dump(outputdir=os.path.join(env.CFG_INVENIO_PREFIX, "tests"))


@task
def test_reset_admin():
    """ Reset admin password """
    puts(cyan(">>> Resetting Invenio admin password..." % env))
    local("""echo "UPDATE user SET password=AES_ENCRYPT(email,'') WHERE nickname='admin';" | %(CFG_INVENIO_PREFIX)s/bin/dbexec""" % env)


@task
def test_load(repo=None):
    """ Load test environment """
    puts(cyan(">>> Loading test package..." % env))
    local("sudo rsync --delete -rLptgoDv %(CFG_INVENIO_PREFIX)s/tests/var/ %(CFG_INVENIO_PREFIX)s/var/" % env)
    local("sudo rsync --delete -rLptgoDv %(CFG_INVENIO_PREFIX)s/tests/etc/ %(CFG_INVENIO_PREFIX)s/etc/" % env)

    mysql_load(dumpfile=os.path.join(env.CFG_INVENIO_PREFIX, "tests/%s.sql.gz" % env.CFG_DATABASE_NAME))
    repo_all_configure_make(repo, target_key='deploy_targets')


# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from fabric.api import prompt, env, local
from jinja2 import Environment, FileSystemLoader
from jinja2.exceptions import TemplateNotFound
import getpass
import os
import re

def prompt_and_check(questions, check_func, cache_key=None, stored_answers=None):
    """
    Ask user for questions, and check answers with supplied function
    """
    done = False

    if stored_answers:
        if check_func(stored_answers):
            answers = stored_answers
            done = True

    if cache_key and cache_key in env.get('answers_cache',{}):
        return env.answers_cache[cache_key]

    while not done:
        answers = {}
        for q, key in questions:
            if 'password' in key:
                answers[key] = getpass.getpass(q + " ")
            else:
                answers[key] = prompt(q)
        if check_func(answers):
            done = True

    if cache_key:
        if 'answers_cache' not in env:
            env.answers_cache = {}
        env.answers_cache[cache_key] = answers
    return answers


def write_template(filename, context, tpl_str=None, tpl_file=None, append=False, mark=None):
    """
    Render template and write output to file

    @param filename: File to write
    @param template: Name of template
    @param context: Dictionary for the template context (usually just env)
    @param append: bool, True if you want to append content to file instead of overwriting.
    @param mark: str, If append is true and mark is defined, then the rendered
                 template will replace any previous appened version.
    """
    if tpl_file:
        try:
            tpl = env.jinja.get_template(tpl_file)
        except TemplateNotFound, e:
            jinja = Environment(loader=FileSystemLoader([os.path.dirname(tpl_file)]))
            tpl = jinja.get_template(os.path.basename(tpl_file))
    elif tpl_str:
        tpl = env.jinja.from_string(tpl_str)
    content = tpl.render(context)
    if append and mark is not None:
        start_mark = "### START: %s ###" % mark
        end_mark = "### END: %s ###" % mark
        marked_content = """%s\n%s\n%s\n""" % (start_mark, content, end_mark)

        file_content = []
        skip = False
        added = False

        f = open(filename, 'r')
        for line in f:
            if line == start_mark:
                skip = True
                added = True
                file_content.append(marked_content)
            elif line == end_mark:
                skip = False
            else:
                if not skip:
                    file_content.append(line)
        f.close()

        if added:
            append = False
            content = "\n".join(file_content)
        else:
            content = marked_content

    f = open(filename, 'a' if append else 'w')
    f.write(content.encode('utf8'))
    f.close()


def python_version():
    """ Determine Python version """
    return local(("%(CFG_INVENIO_PREFIX)s/bin/python -c \"import sys;print str(sys.version_info[0]) + '.' + str(sys.version_info[1])\"") % env, capture=True)


def pythonbrew_versions():
    """ Get all installed Pythonbrew versions """
    pythonsdir = os.path.join(os.environ.get('PYTHONBREW_ROOT',
                          os.path.expanduser('~/.pythonbrew')), 'pythons')

    version_dict = {}
    pat = re.compile("Python-(\d\.\d.\d)")

    for d in os.listdir(pythonsdir):
        pydir = os.path.join(pythonsdir, d)
        pybin = os.path.join(pydir, 'bin/python')
        m = pat.match(d)

        if m and os.path.isdir(pydir) and os.path.exists(pybin):
            major, minor, patch = m.group(1).split('.')
            version_dict["%s%s%s" % (major, minor, patch)] = pybin
            version_dict["%s%s" % (major, minor)] = pybin

    return version_dict



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.


from setuptools import setup, find_packages

setup(
    name='invenio-fabric',
    version='0.2.3',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=False,
    zip_safe=False,
    install_requires=['distribute', 'Fabric>=1.4', 'Jinja2', 'virtualenvwrapper'],
    data_files=[
        ('share/atlantis/',[
            'examples/atlantis/fabfile_advanced.py',
            'examples/atlantis/fabfile_minimal.py',
        ]),
        ('share/atlantis/common/',[
            'examples/atlantis/common/activate-profile.tpl',
            'examples/atlantis/common/requirements.txt',
            'examples/atlantis/common/requirements-extra.txt',
            'examples/atlantis/common/rundevserver.py.tpl',
        ]),
    ],

    author='CERN',
    author_email='lars.holm.nielsen@cern.ch',
    description='Fabric tasks for working with Invenio repository software',
    license="GPL",
    keywords="invenio fabric CERN",
    url="https://github.com/lnielsen-cern/invenio-fabric",
)



from fabric.api import task
from inveniofab.api import *

@task
def loc(activate=True, **kwargs):
    """ Local environment 
    
    Likely you need to do:
    export CFG_SRCDIR="/path/to/src"
    """
    env = env_create('loc', activate=activate, **kwargs)
    return env



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
from fabric.api import local, puts, env, task, abort, warn
from fabric.colors import red, cyan
from fabric.contrib.console import confirm
from inveniofab.utils import write_template
from jinja2.exceptions import TemplateNotFound

#
# Default configuration template
#

INVENIO_LOCAL_TPL = u"""
[Invenio]
CFG_BIBSCHED_PROCESS_USER = {{CFG_INVENIO_USER}}
CFG_DATABASE_HOST = {{CFG_DATABASE_HOST}}
CFG_DATABASE_PORT = {{CFG_DATABASE_PORT|default(3306)}}
CFG_DATABASE_NAME = {{CFG_DATABASE_NAME}}
CFG_DATABASE_USER = {{CFG_DATABASE_USER}}
CFG_DATABASE_PASS = {{CFG_DATABASE_PASS}}
CFG_SITE_URL = http://{{CFG_INVENIO_HOSTNAME}}:{{CFG_INVENIO_PORT_HTTP}}
# For production environments, change http to https in next line
CFG_SITE_SECURE_URL = http://{{CFG_INVENIO_HOSTNAME}}:{{CFG_INVENIO_PORT_HTTPS}}
CFG_SITE_ADMIN_EMAIL = {{CFG_INVENIO_ADMIN}}
CFG_SITE_SUPPORT_EMAIL = {{CFG_INVENIO_ADMIN}}
CFG_SITE_NAME = Atlantis Fictive Institute of Science
CFG_SITE_NAME_INTL_fr = Atlantis Institut des Sciences Fictives
# Next two is only for runnning a debugging mail server
{% if CFG_MISCUTIL_SMTP_HOST and CFG_MISCUTIL_SMTP_PORT %}
CFG_MISCUTIL_SMTP_HOST = {{CFG_MISCUTIL_SMTP_HOST}}
CFG_MISCUTIL_SMTP_PORT = {{CFG_MISCUTIL_SMTP_PORT}}
{% endif %}
CFG_DEVEL_SITE = 0
CFG_SITE_EMERGENCY_EMAIL_ADDRESSES = {'*': '{{CFG_INVENIO_ADMIN}}'}
CFG_WEBSTYLE_INSPECT_TEMPLATES = 0
CFG_FLASK_CACHE_TYPE = redis
"""


@task
def invenio_conf():
    """ Upload and update Invenio configuration """
    puts(cyan(">>> Configuring Invenio..." % env))

    invenio_local = env.get('CFG_INVENIO_CONF', None)
    invenio_local_remote = os.path.join(env.CFG_INVENIO_PREFIX, 'etc/invenio-local.conf')

    if not invenio_local:
        puts(red(">>> CFG_INVENIO_CONF not specified, using built-in template for invenio-local.conf..."))

    puts(">>> Writing invenio-local.conf to %s ..." % invenio_local_remote)
    if not invenio_local:
        write_template(invenio_local_remote, env, tpl_str=INVENIO_LOCAL_TPL)
    else:
        try:
            write_template(invenio_local_remote, env, tpl_file=invenio_local)
        except TemplateNotFound:
            puts(red("Could not find template %s" % invenio_local))
            if not confirm("Use built-in template for invenio-local.conf?"):
                abort("User aborted")
            else:
                write_template(invenio_local_remote, env, tpl_str=INVENIO_LOCAL_TPL)

    inveniocfg("--update-all")


@task
def invenio_upgrade():
    """ Upgrade Invenio """
    puts(cyan(">>> Upgrading Invenio..." % env))
    inveniocfg("--upgrade")


@task
def invenio_createdb():
    """ Create Invenio tables """
    puts(cyan(">>> Creating Invenio tables..." % env))
    inveniocfg("--create-tables")


@task
def invenio_create_demosite():
    """ Create Invenio demo site"""
    puts(cyan(">>> Creating Invenio demo site..." % env))
    inveniocfg("--create-demo-site --load-demo-records")


#
# Helpers
#
def inveniocfg(options):
    """
    Helper to run inveniocfg
    """
    #cmds = " && ".join([]) % env
    local(("%(CFG_INVENIO_PREFIX)s/bin/inveniocfg " % env) + options)



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Fabric tasks for bootstrapping, installing, deploying and running Atlantis.

Examples
--------
Setup virtual environment and database with Python 2.4 and Invenio v1.1.0:
  fab loc:py=24,ref=origin/v1.1.0 bootstrap
  fab loc:py=24,ref=origin/v1.1.0 invenio_create_demosite

Dump, load and drop (database and virtual environment):
  fab loc:py=27,ref=master dump
  fab loc:py=27,ref=master load
  fab loc:py=27,ref=master drop

Requirements:
 * pythonbrew must be installed to specify a specific Python version.
"""

from fabric.api import task, abort
from fabric.colors import red
from inveniofab.api import *
import os

REF_OVERRIDE = {
    'invenio' : {
        'flask' : {
            'bootstrap_targets': ['all', 'install', 'install-mathjax-plugin',
                'install-ckeditor-plugin', 'install-pdfa-helper-files',
                'install-jquery-plugins', 'install-jquery-tokeninput',
                'install-bootstrap' ],
            'deploy_targets': ['all', 'check-upgrade', 'install', ],
            'requirements' : ['%(CFG_INVENIO_SRCDIR)s/requirements.txt', 
                '%(CFG_INVENIO_SRCDIR)s/requirements-extras.txt',
                '%(CFG_INVENIO_SRCDIR)s/requirements-flask.txt',
                '%(CFG_INVENIO_SRCDIR)s/requirements-flask-ext.txt',],
        },
        'origin/v0.99.0' : {
            'bootstrap_targets' : ['all', 'install'],
            'deploy_targets' : ['all', 'install'],
        },
        'origin/v0.99.5' : {
            'bootstrap_targets' : ['all', 'install'],
            'deploy_targets' : ['all', 'install'],
        },
    },
}
"""
Overrides installation procedure for a specific branch.

Certain branches might have special installation requirements which does not
match the default. This dictionary allows you to override the default values.
"""


@task
def loc(activate=True, py=None, ref=None, **kwargs):
    """
    Local environment (example: loc:py=24,ref=maint-1.1)
    """
    env = env_create('loc', name=env_make_name('atlantis', py or '', ref or ''),
                     activate=activate, python=py, **kwargs)

    return env_override(env, 'invenio', ref, global_overrides=REF_OVERRIDE)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 CERN.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

"""
Fabric tasks for defining environments. Specifically it will setup default
values, as well as allow loading an environment without activating it (useful
if you want to say copy a database from production to integration, and thus
needs access to both environments at the same time.
"""

from fabric import state
from fabric.api import execute, abort, task
from fabric.api import task, abort
from inveniofab.utils import pythonbrew_versions
from jinja2 import Environment, FileSystemLoader
import copy
import os

GLOBAL_REFS_OVERRIDE = {
    'invenio' : {
        'next' : {
            'bootstrap_targets': ['all', 'install', 'install-mathjax-plugin',
                'install-ckeditor-plugin', 'install-pdfa-helper-files',
                'install-jquery-plugins', 'install-jquery-tokeninput',
                'install-bootstrap' ],
            'deploy_targets': ['all', 'check-upgrade', 'install', ],
            'requirements' : ['%(CFG_INVENIO_SRCDIR)s/requirements.txt',
                '%(CFG_INVENIO_SRCDIR)s/requirements-extras.txt',
                '%(CFG_INVENIO_SRCDIR)s/requirements-flask.txt',
                '%(CFG_INVENIO_SRCDIR)s/requirements-flask-ext.txt',],
        },
        'origin/v0.99.0' : {
            'bootstrap_targets' : ['all', 'install'],
            'deploy_targets' : ['all', 'install'],
        },
        'origin/v0.99.5' : {
            'bootstrap_targets' : ['all', 'install'],
            'deploy_targets' : ['all', 'install'],
        },
    },
}
"""
Overrides installation procedure for a specific branch.

Certain branches might have special installation requirements which does not
match the default. This dictionary allows you to override the default values.
"""


def env_create(envname, defaults_func=None, activate=True, **kwargs):
    """
    Create a new environment (e.g. integration, production,
    local). Minimal usage::

      @task
      def prod(activate=True, **kwargs)
          env = env_create('prod', activate=activate, **kwargs)

          env.CFG_
          return env

    @param envname: str, Name of environment (must be same as task envname).
    @param defaults_func: callable taking a dictionary as argument and returns
        the same dictionary. Used to setup defaults in the environment.
    @param activate: Bool, True to activate the environment, or False to just
        load (i.e. put config vars in fabric.api.env or not)
    """
    from fabric.api import env

    if activate:
        some_env = env
        some_env.env_active = True
    else:
        tmp = env.jinja
        env.jinja = None

        some_env = copy.deepcopy(env)
        some_env.env_active = False

        env.jinja = tmp

    some_env.env_name = envname

    # Allow user to setup defaults
    if defaults_func:
        some_env = defaults_func(some_env, **kwargs)
    else:
        some_env = env_defaults(some_env, **kwargs)

    return some_env


def env_get(name):
    """
    Get environment by name (does not activate the environment).

    An environment is defined in a local task. This task will
    look for a task with the name <name> and execute. Hence
    the task defining the environment.
    """
    from fabric.api import env

    # Check if active env is requested
    if env.get('env_active', False) and env.get('env_name', None) == name:
        return env

    if 'loaded_envs' not in env:
        env.loaded_envs = {}

    # Check if env is already loaded, if not load it.
    if name not in env.loaded_envs:
        res = execute(name, activate=False)
        env.loaded_envs[name] = res['<local-only>']

    return env.loaded_envs[name]


def env_defaults(env, name='invenio', prefix=None, python=None, **kwargs):
    """
    Setup defaults in environment.

    Note: This can be overridden by the user.
    """
    # Initialise template loader
    env.jinja = Environment(loader=FileSystemLoader([env.env_name, 'common']))
    prefix = prefix or os.path.join(os.getenv('WORKON_HOME', '/opt'), name)

    env.roledefs = {
        'web': [],
        'lb': [],
        'db-master': [],
        'db-slave': [],
        'workers': [],
    }

    if python is None:
        pythonbin = 'python'
    else:
        python_versions = pythonbrew_versions()
        if python in python_versions:
            pythonbin = python_versions[python]
        else:
            python_versions = python_versions.keys()
            python_versions.sort()
            abort("Unknown Python version %s. Available versions are %s" % (
                  python, ", ".join(python_versions)))

    env.update({
        'WITH_VIRTUALENV': True,
        'WITH_DEVSERVER': True,
        'WITH_WORKDIR': True,
        'WITH_DEVSCRIPTS': True,
        'PYTHON': pythonbin,
        'ACTIVATE': '. %(CFG_INVENIO_PREFIX)s/bin/activate',
        'CFG_INVENIO_REPOS': [],
        'CFG_INVENIO_CONF': 'etc/invenio-local.conf',
        'CFG_INVENIO_PREFIX': prefix,
        'CFG_INVENIO_SRCDIR' : os.path.join(prefix, 'src/invenio'),
        'CFG_SRCDIR' : os.environ.get('CFG_SRCDIR', os.path.join(prefix, 'src')),
        # Only use CFG_SRCWORKDIR if WITH_WORKDIR is True
        'CFG_SRCWORKDIR' : os.path.join(prefix, 'src'),
        'CFG_INVENIO_HOSTNAME': "localhost",
        'CFG_INVENIO_DOMAINNAME': "",
        'CFG_INVENIO_PORT_HTTP': "4000",
        'CFG_INVENIO_PORT_HTTPS': "4000",
        'CFG_INVENIO_USER': env.user,
        'CFG_INVENIO_APACHECTL': '/etc/init.d/httpd',
        'CFG_INVENIO_ADMIN': 'nobody@localhost',

        'CFG_DATABASE_DUMPDIR': prefix,
        'CFG_DATABASE_HOST': 'localhost',
        'CFG_DATABASE_PORT': 3306,
        'CFG_DATABASE_NAME': name,
        'CFG_DATABASE_USER': name,
        'CFG_DATABASE_PASS': 'my123p$ss',
        'CFG_DATABASE_DROP_ALLOWED': True,

        # FIXME: Set to localhost or leave empty?
        'CFG_MISCUTIL_SMTP_HOST': '127.0.0.1',
        'CFG_MISCUTIL_SMTP_PORT': '1025',
    })

    env.CFG_INVENIO_REPOS = [
        ('invenio', {
            'repository' : 'http://invenio-software.org/repo/invenio/',
            'ref': 'master',
            'bootstrap_targets': ['all', 'install', 'install-mathjax-plugin', 'install-ckeditor-plugin', 'install-pdfa-helper-files', 'install-jquery-plugins', ],
            'deploy_targets': ['all', 'check-upgrade', 'install', ],
            'requirements': ['requirements.txt', 'requirements-extra.txt', ],
        }),
    ]

    return env


def env_override(env, this_repo, this_ref, override={}, global_overrides=None ):
    """ Override default values for repository """
    if global_overrides:
        try:
            this_repo_dict = global_overrides[this_repo][this_ref]
        except KeyError:
            return env
    elif override:
        this_repo_dict = override
    else:
        try:
            this_repo_dict = GLOBAL_REFS_OVERRIDE[this_repo][this_ref]
        except KeyError:
            return env

    def _mapper(x):
        repo, repo_dict = x

        if repo == this_repo:
            repo_dict.update(this_repo_dict)
            repo_dict['ref'] = this_ref
        return (repo, repo_dict)

    env.CFG_INVENIO_REPOS = map(_mapper, env.CFG_INVENIO_REPOS)

    return env


def env_make_name(prefix, python, ref):
    """ Generate a MySQL friendly environment name. """
    ref = ref.split("/")
    ref = ref[-1].replace("-","_")
    python = python.replace(".","")
    
    prefix = prefix.replace("_","").replace("-","")

    name = "%s%s%s" % (prefix, python, ref)
    if len(name) > 16:
        name = name.replace("_", "", len(name) - 16)
        return name[:16]

    return name
